/* createtd2.c   2006.7 Y.Senta

   Input:  'harmonics.txt' from XTide WebPage
               http://bel-marduk.unh.edu/xtide/files.html
   Output: '*.td2' format files (common format on Japanese PCs)

   This program is very temporize code because I use it only once.
   Unit "Knots" and "Knots^2" are not supported.
*/

#include <stdio.h>
#include <strings.h>

char Buffer[1024];

char *cutspace(char *c){
  while (*c == ' ' || *c == '\t'){
    c++;
  }
  return c;
}

char *cutcrln(char *c){
  char *p=c;
  while (*p){
    if (*p=='\n' || *p=='\r'){
      *p=0;
      break;
    }
    p++;
  }
  return c;
}

char *timestr(char *c){
  static char r[1024];
  int i,j;
  i=0;
  j=0;
  r[0]='+';

  while(c[i]){
    if (c[i]==' ' || c[i]=='\t')
      break;
    if (c[i++]==':')
      continue;
    r[j++]=c[i-1];
  }
  r[j]=0;

  j=atoi(r);
  if (j>=0)
    sprintf(r, "+%04d", j);
  else
    sprintf(r, "-%04d", -j);
  
  return r;
}

char *portstr(char *c){
  static char r[1024];
  int i;
  i=0;

  while (c[i]){
    if (c[i]==',')
      break;
    if (c[i]==' ' || c[i]=='\t')
      r[i]='_';
    else
      r[i]=c[i];
    i++;
  }
  r[i]=0;
  return r;
}

char *pathstr(char *p, char *t){
  static char r[1024];
  char *q;
  int tmp;

  strcpy(r, "T");
  strcat(r, timestr(t));
  strcat(r, "_");
  q=r+strlen(r);

  while (*p){
    if (*p ==' ' || *p == '\t' || *p=='/')
      *q='_';
    else if (*p == '\'' || *p==',' || *p =='.'){
      p++;
      continue;
    } else if (*p < 0) { /* ISO-8859-1 */
      tmp=*p & 0xff;
      if (tmp >=0xc0){
	if (tmp < 0xc6)
	  *q='A';
	else if (tmp ==0xc6){
	  *q='A';
	  q++;
	  *q='E';
	} else if (tmp==0xc7)
	  *q='C';
	else if (tmp < 0xcc)
	  *q='E';
	else if (tmp < 0xd0)
	  *q='I';
	else if (tmp == 0xd0)
	  *q='D';
	else if (tmp == 0xd1)
	  *q='N';
	else if (tmp < 0xd7)
	  *q='O';
/*	else if (tmp == 0xd7)
	  *q='*';*/
	else if (tmp == 0xd8)
	  *q='O';
	else if (tmp < 0xdd)
	  *q='U';
	else if (tmp == 0xdd)
	  *q='Y';
	else if (tmp == 0xde)
	  *q='P';
	else if (tmp == 0xdf)
	  *q='B';

	else if (tmp < 0xe6)
	  *q='a';
	else if (tmp ==0xe6){
	  *q='a';
	  q++;
	  *q='e';
	} else if (tmp==0xe7)
	  *q='c';
	else if (tmp < 0xec)
	  *q='e';
	else if (tmp < 0xe0)
	  *q='i';
	else if (tmp == 0xf0)
	  *q='d';
	else if (tmp == 0xf1)
	  *q='n';
	else if (tmp < 0xf7)
	  *q='o';
/*	else if (tmp == 0xf7)
	  *q='/'; */
	else if (tmp == 0xf8)
	  *q='o';
	else if (tmp < 0xfd)
	  *q='u';
	else if (tmp == 0xfd)
	  *q='y';
	else if (tmp == 0xfe)
	  *q='p';
	else if (tmp == 0xff)
	  *q='y';
      }
    } else {
      *q=*p;
    }
    q++;
    p++;
  }
  *q=0;

  strcat(r, ".TD2");
  
  return r;
}

double degnum(double a){
  int n;
  int flg;
  double min;

  if (a>=0)
    flg=1;
  else
    flg=-1;

  a *= flg;
  
  n=(int)a;
  min = a - (double)n;
  min *= 0.6;

  return flg*((double)n+min);
}

double cmlevel(char *c){
  double a;
  char p[256];

  if (sscanf(c, "%lf %s", &a, p)==0)
    return 0;

  if (!strcmp(p, "meters"))
    a *= 100;
  if (!strcmp(p, "feet"))
    a *= 30.48;

  return a;
}

int dumptd2(FILE *fp, double unit){
  FILE *fout;
  double lng, lat;
  int sum, count;

  char PortName[1024];
  char TimeZone[1024];
  char PortLevel[1024];

  double Sa_1, Sa_2, Ssa_1, Ssa_2, Mm_1, Mm_2, Msf_1, Msf_2;
  double Mf_1, Mf_2, Q1_1, Q1_2, Rho1_1, Rho1_2, O1_1, O1_2;
  double Mp1_1, Mp1_2, M1_1, M1_2, Pi1_1, Pi1_2, P1_1, P1_2;
  double S1_1, S1_2, K1_1, K1_2, Psi1_1, Psi1_2, Phi1_1, Phi1_2;
  double J1_1, J1_2, So1_1, So1_2, Oo1_1, Oo1_2, d2N2_1, d2N2_2;
  double Mu2_1, Mu2_2, N2_1, N2_2, Nu2_1, Nu2_2, Op2_1, Op2_2;
  double M2_1, M2_2, Lam2_1, Lam2_2, L2_1, L2_2, T2_1, T2_2;
  double S2_1, S2_2, R2_1, R2_2, K2_1, K2_2, d2Sm2_1, d2Sm2_2;
  double Mo3_1, Mo3_2, M3_1, M3_2, Mk3_1, Mk3_2, Sk3_1, Sk3_2;
  double M4_1, M4_2, Ms4_1, Ms4_2, M6_1, M6_2, d2Ms6_1, d2Ms6_2;

  int bContinue;

  Sa_1=Sa_2=Ssa_1=Ssa_2=Mm_1=Mm_2=Msf_1=Msf_2=0;
  Mf_1=Mf_2=Q1_1=Q1_2=Rho1_1=Rho1_2=O1_1=O1_2=0;
  Mp1_1=Mp1_2=M1_1=M1_2=Pi1_1=Pi1_2=P1_1=P1_2=0;
  S1_1=S1_2=K1_1=K1_2=Psi1_1=Psi1_2=Phi1_1=Phi1_2=0;
  J1_1=J1_2=So1_1=So1_2=Oo1_1=Oo1_2=d2N2_1=d2N2_2=0;
  Mu2_1=Mu2_2=N2_1=N2_2=Nu2_1=Nu2_2=Op2_1=Op2_2=0;
  M2_1=M2_2=Lam2_1=Lam2_2=L2_1=L2_2=T2_1=T2_2=0;
  S2_1=S2_2=R2_1=R2_2=K2_1=K2_2=d2Sm2_1=d2Sm2_2=0;
  Mo3_1=Mo3_2=M3_1=M3_2=Mk3_1=Mk3_2=Sk3_1=Sk3_2=0;
  M4_1=M4_2=Ms4_1=Ms4_2=M6_1=M6_2=d2Ms6_1=d2Ms6_2=0;

  sum = 0;
  count = 0;
  bContinue=0;
  
  while (fgets(Buffer, sizeof(Buffer), fp)){
    if (!strncmp(Buffer, "# !units:", 9)){
      bContinue=1;
      break;
    }

    if (!strncmp(Buffer, "# !longitude:", 13)){
      if (sscanf(Buffer+13, "%lf", &lng)==1)
	sum++;
      continue;
    }

    if (!strncmp(Buffer, "# !latitude:", 12)){
      if (sscanf(Buffer+12, "%lf", &lat)==1)
	sum++;
      continue;
    }

    if (Buffer[0]!='#')
      count++;

    if (count==1){
      strcpy(PortName, cutcrln(Buffer));
      sum++;
    } else if (count==2){
      strcpy(TimeZone, cutcrln(Buffer));
      sum++;
    } else if (count==3){
      strcpy(PortLevel, cutcrln(Buffer));
      sum++;
    } else {
      if (!strncmp("SA", Buffer, 2)){
	if (sscanf(Buffer+2, "%lf %lf", &Sa_1, &Sa_2)==2)
	  sum+=2;
      } else if (!strncmp("SSA", Buffer, 3)){
	sscanf(Buffer+3, "%lf %lf", &Ssa_1, &Ssa_2);
      } else if (!strncmp("MM", Buffer, 2)){
	sscanf(Buffer+2, "%lf %lf", &Mm_1, &Mm_2);
      } else if (!strncmp("MSF", Buffer, 3)){
	sscanf(Buffer+3, "%lf %lf", &Msf_1, &Msf_2);
      } else if (!strncmp("MF", Buffer, 2)){
	sscanf(Buffer+2, "%lf %lf", &Mf_1, &Mf_2);
      } else if (!strncmp("Q1", Buffer, 2)){
	sscanf(Buffer+2, "%lf %lf", &Q1_1, &Q1_2);
      } else if (!strncmp("RHO1", Buffer, 4)){
	sscanf(Buffer+4, "%lf %lf", &Rho1_1, &Rho1_2);
      } else if (!strncmp("O1", Buffer, 2)){
	if (sscanf(Buffer+2, "%lf %lf", &O1_1, &O1_2)==2)
	  sum+=2;
      } else if (!strncmp("MP1", Buffer, 3)){
	sscanf(Buffer+3, "%lf %lf", &Mp1_1, &Mp1_2);
      } else if (!strncmp("M1", Buffer, 2)){
	sscanf(Buffer+2, "%lf %lf", &M1_1, &M1_2);
      } else if (!strncmp("PI1", Buffer, 3)){
	sscanf(Buffer+3, "%lf %lf", &Pi1_1, &Pi1_2);
      } else if (!strncmp("P1", Buffer, 2)){
	if (sscanf(Buffer+2, "%lf %lf", &P1_1, &P1_2)==2)
	  sum+=2;
      } else if (!strncmp("S1", Buffer, 2)){
	sscanf(Buffer+2, "%lf %lf", &S1_1, &S1_2);
      } else if (!strncmp("K1", Buffer, 2)){
	if (sscanf(Buffer+2, "%lf %lf", &K1_1, &K1_2)==2)
	  sum+=2;
      } else if (!strncmp("PSI1", Buffer, 4)){
	sscanf(Buffer+4, "%lf %lf", &Psi1_1, &Psi1_2);
      } else if (!strncmp("PHI1", Buffer, 4)){
	sscanf(Buffer+4, "%lf %lf", &Phi1_1, &Phi1_2);
      } else if (!strncmp("J1", Buffer, 2)){
	sscanf(Buffer+2, "%lf %lf", &J1_1, &J1_2);
      } else if (!strncmp("SO1", Buffer, 3)){
	sscanf(Buffer+3, "%lf %lf", &So1_1, &So1_2);
      } else if (!strncmp("OO1", Buffer, 3)){
	sscanf(Buffer+3, "%lf %lf", &Oo1_1, &Oo1_2);
      } else if (!strncmp("2N2", Buffer, 3)){
	sscanf(Buffer+3, "%lf %lf", &d2N2_1, &d2N2_2);
      } else if (!strncmp("MU2", Buffer, 3)){
	sscanf(Buffer+3, "%lf %lf", &Mu2_1, &Mu2_2);
      } else if (!strncmp("N2", Buffer, 2)){
	sscanf(Buffer+2, "%lf %lf", &N2_1, &N2_2);
      } else if (!strncmp("NU2", Buffer, 3)){
	sscanf(Buffer+3, "%lf %lf", &Nu2_1, &Nu2_2);
      } else if (!strncmp("OP2", Buffer, 3)){
	sscanf(Buffer+3, "%lf %lf", &Op2_1, &Op2_2);
      } else if (!strncmp("M2", Buffer, 2)){
	if (sscanf(Buffer+2, "%lf %lf", &M2_1, &M2_2)==2)
	  sum+=2;
      } else if (!strncmp("LDA2", Buffer, 4)){  /* I'm not sure */
	sscanf(Buffer+4, "%lf %lf", &Lam2_1, &Lam2_2);
      } else if (!strncmp("L2", Buffer, 2)){
	sscanf(Buffer+2, "%lf %lf", &L2_1, &L2_2);
      } else if (!strncmp("T2", Buffer, 2)){
	sscanf(Buffer+2, "%lf %lf", &T2_1, &T2_2);
      } else if (!strncmp("S2", Buffer, 2)){
	if (sscanf(Buffer+2, "%lf %lf", &S2_1, &S2_2)==2)
	  sum+=2;
      } else if (!strncmp("R2", Buffer, 2)){
	sscanf(Buffer+2, "%lf %lf", &R2_1, &R2_2);
      } else if (!strncmp("K2", Buffer, 2)){
	if (sscanf(Buffer+2, "%lf %lf", &K2_1, &K2_2)==2)
	  sum+=2;
      } else if (!strncmp("2SM2", Buffer, 4)){
	sscanf(Buffer+4, "%lf %lf", &d2Sm2_1, &d2Sm2_2);
      } else if (!strncmp("MO3", Buffer, 3)){
	sscanf(Buffer+3, "%lf %lf", &Mo3_1, &Mo3_2);
      } else if (!strncmp("M3", Buffer, 2)){
	sscanf(Buffer+2, "%lf %lf", &M3_1, &M3_2);
      } else if (!strncmp("MK3", Buffer, 3)){
	sscanf(Buffer+3, "%lf %lf", &Mk3_1, &Mk3_2);
      } else if (!strncmp("SK3", Buffer, 3)){
	sscanf(Buffer+3, "%lf %lf", &Sk3_1, &Sk3_2);
      } else if (!strncmp("M4", Buffer, 2)){
	sscanf(Buffer+2, "%lf %lf", &M4_1, &M4_2);
      } else if (!strncmp("MS4", Buffer, 3)){
	sscanf(Buffer+3, "%lf %lf", &Ms4_1, &Ms4_2);
      } else if (!strncmp("M6", Buffer, 2)){
	sscanf(Buffer+2, "%lf %lf", &M6_1, &M6_2);
      } else if (!strncmp("2MS6", Buffer, 4)){
	sscanf(Buffer+4, "%lf %lf", &d2Ms6_1, &d2Ms6_2);
      }
    }
  }

  if (sum==19){
    fout = fopen(pathstr(PortName, TimeZone), "w");
    if (fout){
      /* DOS (CR+LF) format */
      
      fprintf(fout, "%s, %6.2lf, %6.2lf, %3.0lf\r\n",
	      portstr(PortName), degnum(lat), degnum(lng), cmlevel(PortLevel));
      fprintf(fout, "Sa  , %6.2lf, %6.2lf,      Ssa , %6.2lf, %6.2lf\r\n",
	      Sa_1*unit, Sa_2, Ssa_1*unit, Ssa_2);
      fprintf(fout, "Mm  , %6.2lf, %6.2lf,      MSf , %6.2lf, %6.2lf\r\n",
	      Mm_1*unit, Mm_2, Msf_1*unit, Msf_2);
      fprintf(fout, "Mf  , %6.2lf, %6.2lf,      Q1  , %6.2lf, %6.2lf\r\n",
	      Mf_1*unit, Mf_2, Q1_1*unit, Q1_2);
      fprintf(fout, "Rho1, %6.2lf, %6.2lf,      O1  , %6.2lf, %6.2lf\r\n",
	      Rho1_1*unit, Rho1_2, O1_1*unit, O1_2);
      fprintf(fout, "MP1 , %6.2lf, %6.2lf,      M1  , %6.2lf, %6.2lf\r\n",
	      Mp1_1*unit, Mp1_2, M1_1*unit, M1_2);
      fprintf(fout, "Pi1 , %6.2lf, %6.2lf,      P1  , %6.2lf, %6.2lf\r\n",
	      Pi1_1*unit, Pi1_2, P1_1*unit, P1_2);
      fprintf(fout, "S1  , %6.2lf, %6.2lf,      K1  , %6.2lf, %6.2lf\r\n",
	      S1_1*unit, S1_2, K1_1*unit, K1_2);
      fprintf(fout, "Psi1, %6.2lf, %6.2lf,      Phi1, %6.2lf, %6.2lf\r\n",
	      Psi1_1*unit, Psi1_2, Phi1_1*unit, Phi1_2);
      fprintf(fout, "J1  , %6.2lf, %6.2lf,      SO1 , %6.2lf, %6.2lf\r\n",
	      J1_1*unit, J1_2, So1_1*unit, So1_2);
      fprintf(fout, "OO1 , %6.2lf, %6.2lf,      2N2 , %6.2lf, %6.2lf\r\n",
	      Oo1_1*unit, Oo1_2, d2N2_1*unit, d2N2_2);
      fprintf(fout, "Mu2 , %6.2lf, %6.2lf,      N2  , %6.2lf, %6.2lf\r\n",
	      Mu2_1*unit, Mu2_2, N2_1*unit, N2_2);
      fprintf(fout, "Nu2 , %6.2lf, %6.2lf,      OP2 , %6.2lf, %6.2lf\r\n",
	      Nu2_1*unit, Nu2_2, Op2_1*unit, Op2_2);
      fprintf(fout, "M2  , %6.2lf, %6.2lf,      Lam2, %6.2lf, %6.2lf\r\n",
	      M2_1*unit, M2_2, Lam2_1*unit, Lam2_2);
      fprintf(fout, "L2  , %6.2lf, %6.2lf,      T2  , %6.2lf, %6.2lf\r\n",
	      L2_1*unit, L2_2, T2_1*unit, T2_2);
      fprintf(fout, "S2  , %6.2lf, %6.2lf,      R2  , %6.2lf, %6.2lf\r\n",
	      S2_1*unit, S2_2, R2_1*unit, R2_2);
      fprintf(fout, "K2  , %6.2lf, %6.2lf,      2SM2, %6.2lf, %6.2lf\r\n",
	      K2_1*unit, K2_2, d2Sm2_1*unit, d2Sm2_2);
      fprintf(fout, "MO3 , %6.2lf, %6.2lf,      M3  , %6.2lf, %6.2lf\r\n",
	      Mo3_1*unit, Mo3_2, M3_1*unit, M3_2);
      fprintf(fout, "MK3 , %6.2lf, %6.2lf,      SK3 , %6.2lf, %6.2lf\r\n",
	      Mk3_1*unit, Mk3_2, Sk3_1*unit, Sk3_2);
      fprintf(fout, "M4  , %6.2lf, %6.2lf,      MS4 , %6.2lf, %6.2lf\r\n",
	      M4_1*unit, M4_2, Ms4_1*unit, Ms4_2);
      fprintf(fout, "M6  , %6.2lf, %6.2lf,      2MS6, %6.2lf, %6.2lf\r\n",
	      M6_1*unit, M6_2, d2Ms6_1*unit, d2Ms6_2);
      fclose(fout);
    }
  }

  return bContinue;
}

int main(){
  FILE *fp;
  char *pUnit;
  int bFound;
  int bKnots;
  double cmUnit;

  fp=fopen("harmonics.txt", "r");
  if (!fp){
    fprintf(stderr, "Can not open harmonics.txt\n");
    return 1;
  }


  bFound=0;
  for (;;){
    if (!bFound){
      if (!fgets(Buffer, sizeof(Buffer), fp)){
	break;
      }
      bFound=0;
    }

    if (!strncmp(Buffer, "# !units:", 9)){
      pUnit=cutspace(Buffer+9);
      bKnots=0;

      if (!strncmp(pUnit, "meters", 6))
	cmUnit=100;
      else if (!strncmp(pUnit, "feet", 4))
	cmUnit=30.48;
      else if (!strncmp(pUnit, "knots", 5)) 
	bKnots=1; /* if knots or knots^2 skip */

      if (!dumptd2(fp, cmUnit))
	break;
      bFound=1;
    }
  }

  fclose(fp);
  return 0;
}
